@@ -20,6 +20,7 @@ from tinycss2.color3 import RGBA  # isort:skip
 from tinycss2.color3 import parse_color as parse_color3  # isort:skip
 from tinycss2.color4 import Color  # isort:skip
 from tinycss2.color4 import parse_color as parse_color4  # isort:skip
+from tinycss2.color5 import parse_color as parse_color5  # isort:skip
 from tinycss2.nth import parse_nth  # isort:skip
 
 
@@ -159,20 +160,9 @@ def _number(value):
     return str(int(value) if value.is_integer() else value)
 
 
-def test_color_currentcolor_3():
-    for value in ('currentcolor', 'currentColor', 'CURRENTCOLOR'):
-        assert parse_color3(value) == 'currentColor'
-
-
-def test_color_currentcolor_4():
-    for value in ('currentcolor', 'currentColor', 'CURRENTCOLOR'):
-        assert parse_color4(value) == 'currentcolor'
-
-
-@json_test()
-def test_color_function_4(input):
-    if not (color := parse_color4(input)):
-        return None
+def _build_color(color):
+    if color is None:
+        return
     (*coordinates, alpha) = color
     result = f'color({color.space}'
     for coordinate in coordinates:
@@ -183,6 +173,42 @@ def test_color_function_4(input):
     return result
 
 
+def test_color_currentcolor_3():
+    for value in ('currentcolor', 'currentColor', 'CURRENTCOLOR'):
+        assert parse_color3(value) == 'currentColor'
+
+
+def test_color_currentcolor_4():
+    for value in ('currentcolor', 'currentColor', 'CURRENTCOLOR'):
+        assert parse_color4(value) == 'currentcolor'
+
+
+def test_color_currentcolor_5():
+    for value in ('currentcolor', 'currentColor', 'CURRENTCOLOR'):
+        assert parse_color5(value) == 'currentcolor'
+
+
+@json_test()
+def test_color_function_4(input):
+    return _build_color(parse_color4(input))
+
+
+@json_test(filename='color_function_4.json')
+def test_color_function_4_with_5(input):
+    return _build_color(parse_color5(input))
+
+
+@json_test()
+def test_color_functions_5(input):
+    if input.startswith('light-dark'):
+        result = []
+        result.append(_build_color(parse_color5(input, ('light',))))
+        result.append(_build_color(parse_color5(input, ('dark',))))
+    else:
+        result = _build_color(parse_color5(input))
+    return result
+
+
 @json_test()
 def test_color_hexadecimal_3(input):
     if not (color := parse_color3(input)):
@@ -196,6 +222,19 @@ def test_color_hexadecimal_3(input):
     return result
 
 
+@json_test(filename='color_hexadecimal_3.json')
+def test_color_hexadecimal_3_with_4(input):
+    if not (color := parse_color4(input)):
+        return None
+    (*coordinates, alpha) = color
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_hexadecimal_4(input):
     if not (color := parse_color4(input)):
@@ -210,9 +249,23 @@ def test_color_hexadecimal_4(input):
     return result
 
 
+@json_test(filename='color_hexadecimal_4.json')
+def test_color_hexadecimal_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    assert color.space == 'srgb'
+    (*coordinates, alpha) = color
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test(filename='color_hexadecimal_3.json')
-def test_color_hexadecimal_3_with_4(input):
-    if not (color := parse_color4(input)):
+def test_color_hexadecimal_3_with_5(input):
+    if not (color := parse_color5(input)):
         return None
     assert color.space == 'srgb'
     (*coordinates, alpha) = color
@@ -251,6 +304,20 @@ def test_color_hsl_3_with_4(input):
     return result
 
 
+@json_test(filename='color_hsl_3.json')
+def test_color_hsl_3_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    assert color.space == 'hsl'
+    (*coordinates, alpha) = color.to('srgb')
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_hsl_4(input):
     if not (color := parse_color4(input)):
@@ -265,6 +332,20 @@ def test_color_hsl_4(input):
     return result
 
 
+@json_test(filename='color_hsl_4.json')
+def test_color_hsl_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    assert color.space == 'hsl'
+    (*coordinates, alpha) = color.to('srgb')
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_hwb_4(input):
     if not (color := parse_color4(input)):
@@ -279,6 +360,20 @@ def test_color_hwb_4(input):
     return result
 
 
+@json_test(filename='color_hwb_4.json')
+def test_color_hwb_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    assert color.space == 'hwb'
+    (*coordinates, alpha) = color.to('srgb')
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_keywords_3(input):
     if not (color := parse_color3(input)):
@@ -310,6 +405,22 @@ def test_color_keywords_3_with_4(input):
     return result
 
 
+@json_test(filename='color_keywords_3.json')
+def test_color_keywords_3_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'srgb'
+    (*coordinates, alpha) = color
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_keywords_4(input):
     if not (color := parse_color4(input)):
@@ -326,6 +437,22 @@ def test_color_keywords_4(input):
     return result
 
 
+@json_test(filename='color_keywords_4.json')
+def test_color_keywords_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'srgb'
+    (*coordinates, alpha) = color
+    result = f'rgb{"a" if alpha != 1 else ""}('
+    result += f'{", ".join(_number(coordinate * 255) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f', {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_lab_4(input):
     if not (color := parse_color4(input)):
@@ -342,6 +469,22 @@ def test_color_lab_4(input):
     return result
 
 
+@json_test(filename='color_lab_4.json')
+def test_color_lab_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'lab'
+    (*coordinates, alpha) = color
+    result = f'{color.space}('
+    result += f'{" ".join(_number(coordinate) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f' / {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_oklab_4(input):
     if not (color := parse_color4(input)):
@@ -358,6 +501,22 @@ def test_color_oklab_4(input):
     return result
 
 
+@json_test(filename='color_oklab_4.json')
+def test_color_oklab_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'oklab'
+    (*coordinates, alpha) = color
+    result = f'{color.space}('
+    result += f'{" ".join(_number(coordinate) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f' / {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_lch_4(input):
     if not (color := parse_color4(input)):
@@ -374,6 +533,22 @@ def test_color_lch_4(input):
     return result
 
 
+@json_test(filename='color_lch_4.json')
+def test_color_lch_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'lch'
+    (*coordinates, alpha) = color
+    result = f'{color.space}('
+    result += f'{" ".join(_number(coordinate) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f' / {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_color_oklch_4(input):
     if not (color := parse_color4(input)):
@@ -390,6 +565,22 @@ def test_color_oklch_4(input):
     return result
 
 
+@json_test(filename='color_oklch_4.json')
+def test_color_oklch_4_with_5(input):
+    if not (color := parse_color5(input)):
+        return None
+    elif isinstance(color, str):
+        return color
+    assert color.space == 'oklch'
+    (*coordinates, alpha) = color
+    result = f'{color.space}('
+    result += f'{" ".join(_number(coordinate) for coordinate in coordinates)}'
+    if alpha != 1:
+        result += f' / {_number(alpha)}'
+    result += ')'
+    return result
+
+
 @json_test()
 def test_stylesheet_bytes(kwargs):
     kwargs['css_bytes'] = kwargs['css_bytes'].encode('latin1')
@@ -474,6 +665,27 @@ def test_backslash_delim():
     assert serialize(tokens) == source
 
 
-def test_bad_unicode():
-    parse_one_declaration('background:\udca9')
-    parse_rule_list('@\udca9')
+def test_escape_in_at_rule():
+    at_rule, = parse_rule_list('@\udca9')
+    assert at_rule.type == 'at-rule'
+    assert at_rule.at_keyword == '\udca9'
+
+
+def test_escape_in_ident():
+    declaration = parse_one_declaration('background:\udca9')
+    assert declaration.type == 'declaration'
+    value, = declaration.value
+    assert value.value == '\udca9'
+
+
+def test_escape_in_dimension_token():
+    dimension, = parse_component_value_list('0\\dddf')
+    assert dimension.type == 'dimension'
+    assert dimension.int_value == 0
+    assert dimension.unit == '\udddf'
+
+
+def test_escape_in_function_name():
+    function, = parse_component_value_list('\\dddf()')
+    assert function.type == 'function'
+    assert function.name == '\udddf'
